@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,24 +28,34 @@
 # TODO: "--profile=cum", to change sort order.  Is there any value in leaving
 # the profile output behind so it can be interactively examined?
 
-import codecs
-import errno
 import os
-from warnings import warn
 import sys
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import codecs
+import errno
+from warnings import warn
+
 import bzrlib
-import bzrlib.errors as errors
-from bzrlib.errors import (BzrError,
-                           BzrCommandError,
-                           BzrCheckError,
-                           NotBranchError)
-from bzrlib import option
+from bzrlib import (
+    debug,
+    errors,
+    option,
+    osutils,
+    trace,
+    )
+""")
+
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_method,
+    zero_eight,
+    zero_eleven,
+    )
+# Compatibility
 from bzrlib.option import Option
-import bzrlib.osutils
-from bzrlib.symbol_versioning import (deprecated_method, zero_eight)
-import bzrlib.trace
-from bzrlib.trace import mutter, note, log_error, warning, be_quiet
+
 
 plugin_cmds = {}
 
@@ -66,7 +76,7 @@ def register_command(cmd, decorate=False):
         k_unsquished = k
     if k_unsquished not in plugin_cmds:
         plugin_cmds[k_unsquished] = cmd
-        mutter('registered plugin command %s', k_unsquished)
+        ## trace.mutter('registered plugin command %s', k_unsquished)
         if decorate and k_unsquished in builtin_command_names():
             return _builtin_commands()[k_unsquished]
     elif decorate:
@@ -74,8 +84,8 @@ def register_command(cmd, decorate=False):
         plugin_cmds[k_unsquished] = cmd
         return result
     else:
-        log_error('Two plugins defined the same command: %r' % k)
-        log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
+        trace.log_error('Two plugins defined the same command: %r' % k)
+        trace.log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
 
 
 def _squish_command_name(cmd):
@@ -150,7 +160,7 @@ def get_cmd_object(cmd_name, plugins_override=True):
     if cmd_obj:
         return cmd_obj
 
-    raise BzrCommandError('unknown command "%s"' % cmd_name)
+    raise errors.BzrCommandError('unknown command "%s"' % cmd_name)
 
 
 class Command(object):
@@ -204,6 +214,12 @@ class Command(object):
             replace - put in a bogus character (typically '?')
             exact - do not encode sys.stdout
 
+            NOTE: by default on Windows, sys.stdout is opened as a text
+            stream, therefore LF line-endings are converted to CRLF.
+            When a command uses encoding_type = 'exact', then
+            sys.stdout is forced to be a binary stream, and line-endings
+            will not mangled.
+
     """
     aliases = []
     takes_args = []
@@ -222,10 +238,10 @@ class Command(object):
 
         Maps from long option name to option object."""
         r = dict()
-        r['help'] = Option.OPTIONS['help']
+        r['help'] = option.Option.OPTIONS['help']
         for o in self.takes_options:
             if isinstance(o, basestring):
-                o = Option.OPTIONS[o]
+                o = option.Option.OPTIONS[o]
             r[o.name] = o
         return r
 
@@ -236,10 +252,16 @@ class Command(object):
         # Originally I was using self.stdout, but that looks
         # *way* too much like sys.stdout
         if self.encoding_type == 'exact':
+            # force sys.stdout to be binary stream on win32
+            if sys.platform == 'win32':
+                fileno = getattr(sys.stdout, 'fileno', None)
+                if fileno:
+                    import msvcrt
+                    msvcrt.setmode(fileno(), os.O_BINARY)
             self.outf = sys.stdout
             return
 
-        output_encoding = bzrlib.osutils.get_terminal_encoding()
+        output_encoding = osutils.get_terminal_encoding()
 
         # use 'replace' so that we don't abort if trying to write out
         # in e.g. the default C locale.
@@ -260,7 +282,7 @@ class Command(object):
     def run_argv_aliases(self, argv, alias_argv=None):
         """Parse the command line and run with extra aliases in alias_argv."""
         if argv is None:
-            warn("Passing None for [] is deprecated from bzrlib 0.10", 
+            warn("Passing None for [] is deprecated from bzrlib 0.10",
                  DeprecationWarning, stacklevel=2)
             argv = []
         args, opts = parse_args(self, argv, alias_argv)
@@ -291,7 +313,7 @@ class Command(object):
         shell error code if not.  It's OK for this method to allow
         an exception to raise up.
         """
-        raise NotImplementedError('no implementation of command %r' 
+        raise NotImplementedError('no implementation of command %r'
                                   % self.name())
 
     def help(self):
@@ -316,6 +338,9 @@ class Command(object):
             return None
 
 
+# Technically, this function hasn't been use in a *really* long time
+# but we are only deprecating it now.
+@deprecated_function(zero_eleven)
 def parse_spec(spec):
     """
     >>> parse_spec(None)
@@ -363,7 +388,7 @@ def parse_args(command, argv, alias_argv=None):
         args = argv
 
     options, args = parser.parse_args(args)
-    opts = dict([(k, v) for k, v in options.__dict__.iteritems() if 
+    opts = dict([(k, v) for k, v in options.__dict__.iteritems() if
                  v is not option.OptionParser.DEFAULT_VALUE])
     return args, opts
 
@@ -385,29 +410,29 @@ def _match_argform(cmd, takes_args, args):
                 argdict[argname + '_list'] = None
         elif ap[-1] == '+':
             if not args:
-                raise BzrCommandError("command %r needs one or more %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r needs one or more %s"
+                                             % (cmd, argname.upper()))
             else:
                 argdict[argname + '_list'] = args[:]
                 args = []
         elif ap[-1] == '$': # all but one
             if len(args) < 2:
-                raise BzrCommandError("command %r needs one or more %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r needs one or more %s"
+                                             % (cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
             args[:-1] = []
         else:
             # just a plain arg
             argname = ap
             if not args:
-                raise BzrCommandError("command %r requires argument %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r requires argument %s"
+                               % (cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
             
     if args:
-        raise BzrCommandError("extra argument to command %s: %s"
-                              % (cmd, args[0]))
+        raise errors.BzrCommandError("extra argument to command %s: %s"
+                                     % (cmd, args[0]))
 
     return argdict
 
@@ -450,12 +475,23 @@ def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     return ret
 
 
-def get_alias(cmd):
-    """Return an expanded alias, or None if no alias exists"""
-    import bzrlib.config
-    alias = bzrlib.config.GlobalConfig().get_alias(cmd)
+def get_alias(cmd, config=None):
+    """Return an expanded alias, or None if no alias exists.
+
+    cmd
+        Command to be checked for an alias.
+    config
+        Used to specify an alternative config to use,
+        which is especially useful for testing.
+        If it is unspecified, the global config will be used.
+    """
+    if config is None:
+        import bzrlib.config
+        config = bzrlib.config.GlobalConfig()
+    alias = config.get_alias(cmd)
     if (alias):
-        return alias.split(' ')
+        import shlex
+        return [a.decode('utf-8') for a in shlex.split(alias.encode('utf-8'))]
     return None
 
 
@@ -492,6 +528,7 @@ def run_bzr(argv):
         Run under the Python lsprof profiler.
     """
     argv = list(argv)
+    trace.mutter("bzr arguments: %r", argv)
 
     opt_lsprof = opt_profile = opt_no_plugins = opt_builtin =  \
                 opt_no_aliases = False
@@ -520,7 +557,9 @@ def run_bzr(argv):
         elif a == '--builtin':
             opt_builtin = True
         elif a in ('--quiet', '-q'):
-            be_quiet()
+            trace.be_quiet()
+        elif a.startswith('-D'):
+            debug.debug_flags.add(a[2:])
         else:
             argv_copy.append(a)
         i += 1
@@ -574,7 +613,7 @@ def run_bzr(argv):
         return ret or 0
     finally:
         # reset, in case we may do other commands later within the same process
-        be_quiet(False)
+        trace.be_quiet(False)
 
 def display_command(func):
     """Decorator that suppresses pipe/interrupt errors."""
@@ -602,7 +641,7 @@ def main(argv):
     bzrlib.ui.ui_factory = TextUIFactory()
     argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
     ret = run_bzr_catch_errors(argv)
-    mutter("return code %d", ret)
+    trace.mutter("return code %d", ret)
     return ret
 
 
@@ -614,7 +653,7 @@ def run_bzr_catch_errors(argv):
     except (KeyboardInterrupt, Exception), e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
-        bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)
+        trace.report_exception(sys.exc_info(), sys.stderr)
         if os.environ.get('BZR_PDB'):
             print '**** entering debugger'
             import pdb
