@@ -14,27 +14,32 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import errno
 import os
 import re
-import subprocess
 import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import errno
+import subprocess
 import tempfile
 import time
 
 from bzrlib import (
     errors,
     osutils,
+    patiencediff,
+    textfile,
     )
+""")
+
 # compatability - plugins import compare_trees from diff!!!
 # deprecated as of 0.10
 from bzrlib.delta import compare_trees
-from bzrlib.errors import BzrError
-from bzrlib.patiencediff import unified_diff
-import bzrlib.patiencediff
-from bzrlib.symbol_versioning import (deprecated_function,
-        zero_eight)
-from bzrlib.textfile import check_text_lines
+from bzrlib.symbol_versioning import (
+        deprecated_function,
+        zero_eight,
+        )
 from bzrlib.trace import mutter, warning
 
 
@@ -62,12 +67,12 @@ def internal_diff(old_filename, oldlines, new_filename, newlines, to_file,
         return
     
     if allow_binary is False:
-        check_text_lines(oldlines)
-        check_text_lines(newlines)
+        textfile.check_text_lines(oldlines)
+        textfile.check_text_lines(newlines)
 
     if sequence_matcher is None:
-        sequence_matcher = bzrlib.patiencediff.PatienceSequenceMatcher
-    ud = unified_diff(oldlines, newlines,
+        sequence_matcher = patiencediff.PatienceSequenceMatcher
+    ud = patiencediff.unified_diff(oldlines, newlines,
                       fromfile=old_filename.encode(path_encoding),
                       tofile=new_filename.encode(path_encoding),
                       sequencematcher=sequence_matcher)
@@ -91,9 +96,9 @@ def internal_diff(old_filename, oldlines, new_filename, newlines, to_file,
 
 
 def _set_lang_C():
-    """Set the env var LANG=C"""
+    """Set the env vars LANG=C and LC_ALL=C."""
     osutils.set_or_unset_env('LANG', 'C')
-    osutils.set_or_unset_env('LC_ALL', None)
+    osutils.set_or_unset_env('LC_ALL', 'C')
     osutils.set_or_unset_env('LC_CTYPE', None)
     osutils.set_or_unset_env('LANGUAGE', None)
 
@@ -102,15 +107,15 @@ def _spawn_external_diff(diffcmd, capture_errors=True):
     """Spawn the externall diff process, and return the child handle.
 
     :param diffcmd: The command list to spawn
-    :param capture_errors: Capture stderr as well as setting LANG=C.
-        This lets us read and understand the output of diff, and respond 
-        to any errors.
+    :param capture_errors: Capture stderr as well as setting LANG=C
+        and LC_ALL=C. This lets us read and understand the output of diff,
+        and respond to any errors.
     :return: A Popen object.
     """
     if capture_errors:
         if sys.platform == 'win32':
             # Win32 doesn't support preexec_fn, but that is
-            # okay, because it doesn't support LANG either.
+            # okay, because it doesn't support LANG and LC_ALL either.
             preexec_fn = None
         else:
             preexec_fn = _set_lang_C
@@ -212,16 +217,17 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
             # Write out the new i18n diff response
             to_file.write(out+'\n')
             if pipe.returncode != 2:
-                raise BzrError('external diff failed with exit code 2'
-                               ' when run with LANG=C, but not when run'
-                               ' natively: %r' % (diffcmd,))
+                raise errors.BzrError(
+                               'external diff failed with exit code 2'
+                               ' when run with LANG=C and LC_ALL=C,'
+                               ' but not when run natively: %r' % (diffcmd,))
 
             first_line = lang_c_out.split('\n', 1)[0]
             # Starting with diffutils 2.8.4 the word "binary" was dropped.
             m = re.match('^(binary )?files.*differ$', first_line, re.I)
             if m is None:
-                raise BzrError('external diff failed with exit code 2;'
-                               ' command: %r' % (diffcmd,))
+                raise errors.BzrError('external diff failed with exit code 2;'
+                                      ' command: %r' % (diffcmd,))
             else:
                 # Binary files differ, just return
                 return
@@ -236,8 +242,8 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
             else:
                 msg = 'exit code %d' % rc
                 
-            raise BzrError('external diff failed with %s; command: %r' 
-                           % (rc, diffcmd))
+            raise errors.BzrError('external diff failed with %s; command: %r' 
+                                  % (rc, diffcmd))
 
 
     finally:
@@ -300,29 +306,41 @@ def show_diff(b, from_spec, specific_files, external_diff_options=None,
 
 def diff_cmd_helper(tree, specific_files, external_diff_options, 
                     old_revision_spec=None, new_revision_spec=None,
+                    revision_specs=None,
                     old_label='a/', new_label='b/'):
     """Helper for cmd_diff.
 
-   tree 
+    :param tree:
         A WorkingTree
 
-    specific_files
+    :param specific_files:
         The specific files to compare, or None
 
-    external_diff_options
+    :param external_diff_options:
         If non-None, run an external diff, and pass it these options
 
-    old_revision_spec
+    :param old_revision_spec:
         If None, use basis tree as old revision, otherwise use the tree for
         the specified revision. 
 
-    new_revision_spec
+    :param new_revision_spec:
         If None, use working tree as new revision, otherwise use the tree for
         the specified revision.
     
+    :param revision_specs: 
+        Zero, one or two RevisionSpecs from the command line, saying what revisions 
+        to compare.  This can be passed as an alternative to the old_revision_spec 
+        and new_revision_spec parameters.
+
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
+
+    # TODO: perhaps remove the old parameters old_revision_spec and
+    # new_revision_spec, since this is only really for use from cmd_diff and
+    # it now always passes through a sequence of revision_specs -- mbp
+    # 20061221
+
     def spec_tree(spec):
         if tree:
             revision = spec.in_store(tree.branch)
@@ -331,15 +349,26 @@ def diff_cmd_helper(tree, specific_files, external_diff_options,
         revision_id = revision.rev_id
         branch = revision.branch
         return branch.repository.revision_tree(revision_id)
+
+    if revision_specs is not None:
+        assert (old_revision_spec is None
+                and new_revision_spec is None)
+        if len(revision_specs) > 0:
+            old_revision_spec = revision_specs[0]
+        if len(revision_specs) > 1:
+            new_revision_spec = revision_specs[1]
+
     if old_revision_spec is None:
         old_tree = tree.basis_tree()
     else:
         old_tree = spec_tree(old_revision_spec)
 
-    if new_revision_spec is None:
+    if (new_revision_spec is None
+        or new_revision_spec.spec is None):
         new_tree = tree
     else:
         new_tree = spec_tree(new_revision_spec)
+
     if new_tree is not tree:
         extra_trees = (tree,)
     else:
