@@ -96,8 +96,11 @@ Example usage:
 
 import os
 import time
-from StringIO import StringIO
+from cStringIO import StringIO
 
+from bzrlib import (
+    errors,
+    )
 import bzrlib.config
 from bzrlib.errors import (
         DirectoryNotEmpty,
@@ -111,10 +114,12 @@ from bzrlib.errors import (
         ResourceBusy,
         UnlockableTransport,
         )
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, note
 from bzrlib.transport import Transport
-from bzrlib.osutils import rand_chars
+from bzrlib.osutils import rand_chars, format_delta
 from bzrlib.rio import read_stanza, Stanza
+import bzrlib.ui
+
 
 # XXX: At the moment there is no consideration of thread safety on LockDir
 # objects.  This should perhaps be updated - e.g. if two threads try to take a
@@ -130,8 +135,10 @@ from bzrlib.rio import read_stanza, Stanza
 # TODO: Make sure to pass the right file and directory mode bits to all
 # files/dirs created.
 
+
 _DEFAULT_TIMEOUT_SECONDS = 300
-_DEFAULT_POLL_SECONDS = 0.5
+_DEFAULT_POLL_SECONDS = 1.0
+
 
 class LockDir(object):
     """Write-lock guarding access to data."""
@@ -160,6 +167,8 @@ class LockDir(object):
         self._dir_modebits = dir_modebits
         self.nonce = rand_chars(20)
 
+        self._report_function = note
+
     def __repr__(self):
         return '%s(%s%s)' % (self.__class__.__name__,
                              self.transport.base,
@@ -243,12 +252,8 @@ class LockDir(object):
         self._check_not_locked()
         holder_info = self.peek()
         if holder_info is not None:
-            if bzrlib.ui.ui_factory.get_boolean(
-                "Break lock %s held by %s@%s [process #%s]" % (
-                    self.transport,
-                    holder_info["user"],
-                    holder_info["hostname"],
-                    holder_info["pid"])):
+            lock_info = '\n'.join(self._format_lock_info(holder_info))
+            if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
                 self.force_break(holder_info)
         
     def force_break(self, dead_holder_info):
@@ -341,19 +346,22 @@ class LockDir(object):
         import socket
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
+        try:
+            user = config.user_email()
+        except errors.NoEmailInUsername:
+            user = config.username()
         s = Stanza(hostname=socket.gethostname(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),
                    nonce=self.nonce,
-                   user=config.user_email(),
+                   user=user,
                    )
         return s.to_string()
 
     def _parse_info(self, info_file):
         return read_stanza(info_file.readlines()).as_dict()
 
-    def wait_lock(self, timeout=_DEFAULT_TIMEOUT_SECONDS,
-                  poll=_DEFAULT_POLL_SECONDS):
+    def wait_lock(self, timeout=None, poll=None):
         """Wait a certain period for a lock.
 
         If the lock can be acquired within the bounded time, it
@@ -362,15 +370,44 @@ class LockDir(object):
         approximately `timeout` seconds.  (It may be a bit more if
         a transport operation takes a long time to complete.)
         """
+        if timeout is None:
+            timeout = _DEFAULT_TIMEOUT_SECONDS
+        if poll is None:
+            poll = _DEFAULT_POLL_SECONDS
+
         # XXX: the transport interface doesn't let us guard 
         # against operations there taking a long time.
         deadline = time.time() + timeout
+        deadline_str = None
+        last_info = None
         while True:
             try:
                 self.attempt_lock()
                 return
             except LockContention:
                 pass
+            new_info = self.peek()
+            mutter('last_info: %s, new info: %s', last_info, new_info)
+            if new_info is not None and new_info != last_info:
+                if last_info is None:
+                    start = 'Unable to obtain'
+                else:
+                    start = 'Lock owner changed for'
+                last_info = new_info
+                formatted_info = self._format_lock_info(new_info)
+                if deadline_str is None:
+                    deadline_str = time.strftime('%H:%M:%S',
+                                                 time.localtime(deadline))
+                self._report_function('%s %s\n'
+                                      '%s\n' # held by
+                                      '%s\n' # locked ... ago
+                                      'Will continue to try until %s\n',
+                                      start,
+                                      formatted_info[0],
+                                      formatted_info[1],
+                                      formatted_info[2],
+                                      deadline_str)
+
             if time.time() + poll < deadline:
                 time.sleep(poll)
             else:
@@ -378,7 +415,7 @@ class LockDir(object):
 
     def lock_write(self):
         """Wait for and acquire the lock."""
-        self.attempt_lock()
+        self.wait_lock()
 
     def lock_read(self):
         """Compatibility-mode shared lock.
@@ -408,3 +445,13 @@ class LockDir(object):
             else:
                 raise LockContention(self)
 
+    def _format_lock_info(self, info):
+        """Turn the contents of peek() into something for the user"""
+        lock_url = self.transport.abspath(self.path)
+        delta = time.time() - int(info['start_time'])
+        return [
+            'lock %s' % (lock_url,),
+            'held by %(user)s on host %(hostname)s [process #%(pid)s]' % info,
+            'locked %s' % (format_delta(delta),),
+            ]
+
