@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@ import os
 import re
 import sys
 
+from bzrlib import (
+    ignores,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import BzrCommandError
@@ -35,10 +38,20 @@ class TestCommit(ExternalBase):
         # If forced, it should succeed, but this is not tested here.
         self.run_bzr("init")
         self.build_tree(['hello.txt'])
-        result = self.run_bzr("commit", "-m", "empty", retcode=3)
-        self.assertEqual(('', 'bzr: ERROR: no changes to commit.'
-                              ' use --unchanged to commit anyhow\n'),
-                         result)
+        out,err = self.run_bzr("commit", "-m", "empty", retcode=3)
+        self.assertEqual('', out)
+        self.assertStartsWith(err, 'bzr: ERROR: no changes to commit.'
+                                  ' use --unchanged to commit anyhow\n')
+
+    def test_commit_success(self):
+        """Successful commit should not leave behind a bzr-commit-* file"""
+        self.run_bzr("init")
+        self.run_bzr("commit", "--unchanged", "-m", "message")
+        self.assertEqual('', self.capture('unknowns'))
+
+        # same for unicode messages
+        self.run_bzr("commit", "--unchanged", "-m", u'foo\xb5')
+        self.assertEqual('', self.capture('unknowns'))
 
     def test_commit_with_path(self):
         """Commit tree with path of root specified"""
@@ -300,3 +313,48 @@ class TestCommit(ExternalBase):
         self.assertNotContainsRe(result, 'file-a')
         result = self.run_bzr('status')[0]
         self.assertContainsRe(result, 'removed:\n  file-a')
+
+    def test_strict_commit(self):
+        """Commit with --strict works if everything is known"""
+        ignores._set_user_ignores([])
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add('a')
+        # A simple change should just work
+        self.run_bzr('commit', '--strict', '-m', 'adding a',
+                     working_dir='tree')
+
+    def test_strict_commit_no_changes(self):
+        """commit --strict gives "no changes" if there is nothing to commit"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add('a')
+        tree.commit('adding a')
+
+        # With no changes, it should just be 'no changes'
+        # Make sure that commit is failing because there is nothing to do
+        self.run_bzr_error(['no changes to commit'],
+                           'commit', '--strict', '-m', 'no changes',
+                           working_dir='tree')
+
+        # But --strict doesn't care if you supply --unchanged
+        self.run_bzr('commit', '--strict', '--unchanged', '-m', 'no changes',
+                     working_dir='tree')
+
+    def test_strict_commit_unknown(self):
+        """commit --strict fails if a file is unknown"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add('a')
+        tree.commit('adding a')
+
+        # Add one file so there is a change, but forget the other
+        self.build_tree(['tree/b', 'tree/c'])
+        tree.add('b')
+        self.run_bzr_error(['Commit refused because there are unknown files'],
+                           'commit', '--strict', '-m', 'add b',
+                           working_dir='tree')
+
+        # --no-strict overrides --strict
+        self.run_bzr('commit', '--strict', '-m', 'add b', '--no-strict',
+                     working_dir='tree')
