@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,9 +32,29 @@ class TestHelp(ExternalBase):
                 self.fail("bad output from bzr %s:\n%r" % (cmd, output))
         # see https://launchpad.net/products/bzr/+bug/35940, -h doesn't work
 
+    def test_help_topics(self):
+        """Smoketest for 'bzr help topics'"""
+        out, err = self.run_bzr('help', 'topics')
+        self.assertContainsRe(out, 'basic')
+        self.assertContainsRe(out, 'topics')
+        self.assertContainsRe(out, 'commands')
+        self.assertContainsRe(out, 'revisionspec')
+
+    def test_help_revisionspec(self):
+        """Smoke test for 'bzr help revisionspec'"""
+        out, err = self.run_bzr('help', 'revisionspec')
+        self.assertContainsRe(out, 'revno:')
+        self.assertContainsRe(out, 'date:')
+        self.assertContainsRe(out, 'revid:')
+        self.assertContainsRe(out, 'last:')
+        self.assertContainsRe(out, 'before:')
+        self.assertContainsRe(out, 'ancestor:')
+        self.assertContainsRe(out, 'branch:')
+
     def test_help_commands(self):
         dash_help  = self.runbzr('--help commands')[0]
         commands   = self.runbzr('help commands')[0]
+        hidden = self.runbzr('help hidden-commands')[0]
         long_help  = self.runbzr('help --long')[0]
         qmark_long = self.runbzr('? --long')[0]
         qmark_cmds = self.runbzr('? commands')[0]
@@ -43,6 +63,14 @@ class TestHelp(ExternalBase):
         self.assertEquals(dash_help, qmark_long)
         self.assertEquals(dash_help, qmark_cmds)
 
+    def test_hidden(self):
+        commands = self.runbzr('help commands')[0]
+        hidden = self.runbzr('help hidden-commands')[0]
+        self.assertTrue('commit' in commands)
+        self.assertTrue('commit' not in hidden)
+        self.assertTrue('rocks' in hidden)
+        self.assertTrue('rocks' not in commands)
+
     def test_help_detail(self):
         dash_h  = self.runbzr('commit -h')[0]
         help_x  = self.runbzr('help commit')[0]
