@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -197,3 +197,12 @@ class TestUpdate(ExternalBase):
 
         # The pending merges should still be there
         self.assertEqual(['o2'], checkout1.get_parent_ids()[1:])
+
+    def test_readonly_lightweight_update(self):
+        """Update a light checkout of a readonly branch"""
+        tree = self.make_branch_and_tree('branch')
+        readonly_branch = branch.Branch.open(self.get_readonly_url('branch'))
+        checkout = readonly_branch.create_checkout('checkout', 
+                                                   lightweight=True)
+        tree.commit('empty commit')
+        self.runbzr(['update', 'checkout'])
