@@ -17,22 +17,30 @@
 """Tests for bzrdir implementations - tests a bzrdir format."""
 
 from cStringIO import StringIO
+import errno
 import os
 from stat import S_ISDIR
 import sys
 
 import bzrlib.branch
-import bzrlib.bzrdir as bzrdir
+from bzrlib import (
+    bzrdir,
+    errors,
+    lockdir,
+    repository,
+    transactions,
+    transport,
+    ui,
+    workingtree,
+    )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.check import check
-import bzrlib.errors as errors
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
                            UninitializableFormat,
                            NotBranchError,
                            )
-import bzrlib.repository as repository
 import bzrlib.revision
 from bzrlib.tests import (
                           ChrootedTestCase,
@@ -41,12 +49,8 @@ from bzrlib.tests import (
                           TestSkipped,
                           )
 from bzrlib.trace import mutter
-import bzrlib.transactions as transactions
-import bzrlib.transport as transport
 from bzrlib.transport import get_transport
-import bzrlib.ui as ui
 from bzrlib.upgrade import upgrade
-import bzrlib.workingtree as workingtree
 
 
 class TestCaseWithBzrDir(TestCaseWithTransport):
@@ -150,6 +154,24 @@ class TestBzrDir(TestCaseWithBzrDir):
         wt = dir.create_workingtree(revision_id=bzrlib.revision.NULL_REVISION)
         self.assertEqual([], wt.get_parent_ids())
 
+    def test_destroy_workingtree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        tree.add('file')
+        tree.commit('first commit')
+        bzrdir = tree.bzrdir
+        try:
+            bzrdir.destroy_workingtree()
+        except errors.UnsupportedOperation:
+            raise TestSkipped('Format does not support destroying tree')
+        self.failIfExists('tree/file')
+        self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
+        bzrdir.create_workingtree()
+        self.failUnlessExists('tree/file')
+        bzrdir.destroy_workingtree_metadata()
+        self.failUnlessExists('tree/file')
+        self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
+            
     def test_clone_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'))
@@ -515,11 +537,28 @@ class TestBzrDir(TestCaseWithBzrDir):
         repo = dir.create_repository()
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
+        try:
+            self.assertIs(dir.open_branch().last_revision(), None)
+        except errors.NotBranchError:
+            pass
         target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
+        # testing inventory isn't reasonable for repositories
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/repository/inventory.knit',
+                                     './.bzr/inventory'
                                      ])
+        try:
+            # If we happen to have a tree, we'll guarantee everything
+            # except for the tree root is the same.
+            inventory_f = file(dir.transport.base+'inventory', 'rb')
+            self.assertContainsRe(inventory_f.read(), 
+                                  '<inventory file_id="TREE_ROOT[^"]*"'
+                                  ' format="5">\n</inventory>\n')
+            inventory_f.close()
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
 
     def test_sprout_bzrdir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
@@ -1222,15 +1261,10 @@ class TestBzrDir(TestCaseWithBzrDir):
         dir = self.make_bzrdir('.')
         if dir.can_convert_format():
             # if its default updatable there must be an updater 
-            # (we change the default to match the lastest known format
-            # as downgrades may not be available
-            old_format = bzrdir.BzrDirFormat.get_default_format()
-            bzrdir.BzrDirFormat.set_default_format(dir._format)
-            try:
-                self.assertTrue(isinstance(dir._format.get_converter(),
-                                           bzrdir.Converter))
-            finally:
-                bzrdir.BzrDirFormat.set_default_format(old_format)
+            # (we force the latest known format as downgrades may not be
+            # available
+            self.assertTrue(isinstance(dir._format.get_converter(
+                format=dir._format), bzrdir.Converter))
         dir.needs_format_conversion(None)
 
     def test_upgrade_new_instance(self):
@@ -1242,15 +1276,12 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.createWorkingTreeOrSkip(dir)
         if dir.can_convert_format():
             # if its default updatable there must be an updater 
-            # (we change the default to match the lastest known format
-            # as downgrades may not be available
-            old_format = bzrdir.BzrDirFormat.get_default_format()
-            bzrdir.BzrDirFormat.set_default_format(dir._format)
+            # (we force the latest known format as downgrades may not be
+            # available
             pb = ui.ui_factory.nested_progress_bar()
             try:
-                dir._format.get_converter(None).convert(dir, pb)
+                dir._format.get_converter(format=dir._format).convert(dir, pb)
             finally:
-                bzrdir.BzrDirFormat.set_default_format(old_format)
                 pb.finished()
             # and it should pass 'check' now.
             check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)
@@ -1317,23 +1348,30 @@ class TestBreakLock(TestCaseWithBzrDir):
         unused_repo = thisdir.create_repository()
         master.lock_write()
         unused_repo.lock_write()
-        # two yes's : branch and repository. If the repo in this
-        # dir is inappropriately accessed, 3 will be needed, and
-        # we'll see that because the stream will be fully consumed
-        bzrlib.ui.ui_factory.stdin = StringIO("y\ny\ny\n")
-        master.bzrdir.break_lock()
-        # only two ys should have been read
-        self.assertEqual("y\n", bzrlib.ui.ui_factory.stdin.read())
-        # we should be able to lock a newly opened branch now
-        branch = master.bzrdir.open_branch()
-        branch.lock_write()
-        branch.unlock()
-        # we should not be able to lock the repository in thisdir as its still
-        # held by the explicit lock we took, and the break lock should not have
-        # touched it.
-        repo = thisdir.open_repository()
-        self.assertRaises(errors.LockContention, repo.lock_write)
-        unused_repo.unlock()
+        try:
+            # two yes's : branch and repository. If the repo in this
+            # dir is inappropriately accessed, 3 will be needed, and
+            # we'll see that because the stream will be fully consumed
+            bzrlib.ui.ui_factory.stdin = StringIO("y\ny\ny\n")
+            master.bzrdir.break_lock()
+            # only two ys should have been read
+            self.assertEqual("y\n", bzrlib.ui.ui_factory.stdin.read())
+            # we should be able to lock a newly opened branch now
+            branch = master.bzrdir.open_branch()
+            branch.lock_write()
+            branch.unlock()
+            # we should not be able to lock the repository in thisdir as its still
+            # held by the explicit lock we took, and the break lock should not have
+            # touched it.
+            repo = thisdir.open_repository()
+            orig_default = lockdir._DEFAULT_TIMEOUT_SECONDS
+            try:
+                lockdir._DEFAULT_TIMEOUT_SECONDS = 1
+                self.assertRaises(errors.LockContention, repo.lock_write)
+            finally:
+                lockdir._DEFAULT_TIMEOUT_SECONDS = orig_default
+        finally:
+            unused_repo.unlock()
         self.assertRaises(errors.LockBroken, master.unlock)
 
     def test_break_lock_tree(self):
