@@ -19,6 +19,8 @@ import stat
 import sys
 
 from bzrlib import (
+    errors,
+    generate_ids,
     tests,
     urlutils,
     )
@@ -47,7 +49,7 @@ class TestTreeTransform(TestCaseInTempDir):
     def get_transform(self):
         transform = TreeTransform(self.wt)
         #self.addCleanup(transform.finalize)
-        return transform, transform.trans_id_tree_file_id(self.wt.get_root_id())
+        return transform, transform.root
 
     def test_existing_limbo(self):
         limbo_name = urlutils.local_path_from_url(
@@ -197,7 +199,7 @@ class TestTreeTransform(TestCaseInTempDir):
         transform3.delete_contents(oz_id)
         self.assertEqual(transform3.find_conflicts(), 
                          [('missing parent', oz_id)])
-        root_id = transform3.trans_id_tree_file_id('TREE_ROOT')
+        root_id = transform3.root
         tip_id = transform3.trans_id_tree_file_id('tip-id')
         transform3.adjust_path('tip', root_id, tip_id)
         transform3.apply()
@@ -229,7 +231,7 @@ class TestTreeTransform(TestCaseInTempDir):
     def test_name_invariants(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.new_file('name2', root, 'hello2', 'name2')
         ddir = create_tree.new_directory('dying_directory', root, 'ddir')
@@ -239,7 +241,7 @@ class TestTreeTransform(TestCaseInTempDir):
         create_tree.apply()
 
         mangle_tree,root = self.get_transform()
-        root = mangle_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = mangle_tree.root
         #swap names
         name1 = mangle_tree.trans_id_tree_file_id('name1')
         name2 = mangle_tree.trans_id_tree_file_id('name2')
@@ -324,7 +326,7 @@ class TestTreeTransform(TestCaseInTempDir):
     def test_move_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents, root = self.get_transform()
@@ -340,7 +342,7 @@ class TestTreeTransform(TestCaseInTempDir):
     def test_replace_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents = TreeTransform(self.wt)
@@ -587,10 +589,11 @@ class TestTreeTransform(TestCaseInTempDir):
 
 
 class TransformGroup(object):
-    def __init__(self, dirname):
+    def __init__(self, dirname, root_id):
         self.name = dirname
         os.mkdir(dirname)
         self.wt = BzrDir.create_standalone_workingtree(dirname)
+        self.wt.set_root_id(root_id)
         self.b = self.wt.branch
         self.tt = TreeTransform(self.wt)
         self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
@@ -602,7 +605,8 @@ def conflict_text(tree, merge):
 
 class TestTransformMerge(TestCaseInTempDir):
     def test_text_merge(self):
-        base = TransformGroup("base")
+        root_id = generate_ids.gen_root_id()
+        base = TransformGroup("base", root_id)
         base.tt.new_file('a', base.root, 'a\nb\nc\nd\be\n', 'a')
         base.tt.new_file('b', base.root, 'b1', 'b')
         base.tt.new_file('c', base.root, 'c', 'c')
@@ -612,7 +616,7 @@ class TestTransformMerge(TestCaseInTempDir):
         base.tt.new_directory('g', base.root, 'g')
         base.tt.new_directory('h', base.root, 'h')
         base.tt.apply()
-        other = TransformGroup("other")
+        other = TransformGroup("other", root_id)
         other.tt.new_file('a', other.root, 'y\nb\nc\nd\be\n', 'a')
         other.tt.new_file('b', other.root, 'b2', 'b')
         other.tt.new_file('c', other.root, 'c2', 'c')
@@ -623,7 +627,7 @@ class TestTransformMerge(TestCaseInTempDir):
         other.tt.new_file('h', other.root, 'h\ni\nj\nk\n', 'h')
         other.tt.new_file('i', other.root, 'h\ni\nj\nk\n', 'i')
         other.tt.apply()
-        this = TransformGroup("this")
+        this = TransformGroup("this", root_id)
         this.tt.new_file('a', this.root, 'a\nb\nc\nd\bz\n', 'a')
         this.tt.new_file('b', this.root, 'b', 'b')
         this.tt.new_file('c', this.root, 'c', 'c')
@@ -680,9 +684,10 @@ class TestTransformMerge(TestCaseInTempDir):
     def test_file_merge(self):
         if not has_symlinks():
             raise TestSkipped('Symlinks are not supported on this platform')
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = generate_ids.gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         for tg in this, base, other:
             tg.tt.new_directory('a', tg.root, 'a')
             tg.tt.new_symlink('b', tg.root, 'b', 'b')
@@ -720,9 +725,10 @@ class TestTransformMerge(TestCaseInTempDir):
         self.assertIs(os.path.lexists(this.wt.abspath('h.OTHER')), True)
 
     def test_filename_merge(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = generate_ids.gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
@@ -752,9 +758,10 @@ class TestTransformMerge(TestCaseInTempDir):
         self.assertEqual(this.wt.id2path('f'), pathjoin('b/f1'))
 
     def test_filename_merge_conflicts(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = generate_ids.gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
@@ -904,8 +911,8 @@ class TestBuildTree(tests.TestCaseWithTransport):
         target = self.make_branch_and_tree('target')
         self.build_tree(['target/name'])
         target.add('name')
-        self.assertRaises(AssertionError, build_tree, source.basis_tree(),
-                          target)
+        self.assertRaises(errors.WorkingTreeAlreadyPopulated, 
+            build_tree, source.basis_tree(), target)
 
 
 class MockTransform(object):
